package com.sleepycat.je.cleaner;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.config.EnvironmentParams;
import com.sleepycat.je.dbi.DbConfigManager;
import com.sleepycat.je.dbi.EnvironmentImpl;

public aspect CleanerDaemonAspect {
	private CleanerDaemon EnvironmentImpl.cleanerDaemon;
	
	public CleanerDaemon EnvironmentImpl.getCleanerDaemon() {
		return cleanerDaemon;
	}
	
	after(EnvironmentImpl env) throws DatabaseException:
		execution(void EnvironmentImpl.createDaemons()) && this(env){
		
		/* The cleaner is not time-based so no wakeup interval is used. */
		env.cleanerDaemon = new CleanerDaemon(env, "CleanerDaemon",env.getCleaner());
	}
	
	before(EnvironmentImpl env,DbConfigManager mgr) throws DatabaseException:
		execution(void EnvironmentImpl.runOrPauseDaemons(DbConfigManager)) && this(env) && args(mgr) {
		
		if (!env.isReadOnly()){
			/* Cleaner. Do not start it if running in-memory */
			env.getCleanerDaemon().runOrPause(mgr
					.getBoolean(EnvironmentParams.ENV_RUN_CLEANER)
					&& !mgr.getBoolean(EnvironmentParams.LOG_MEMORY_ONLY));
		}
	}
	after(EnvironmentImpl env):
		execution(void EnvironmentImpl.requestShutdownDaemons()) && this(env){
		
		if (env.cleanerDaemon != null) {
			env.cleanerDaemon.requestShutdown();
		}
	}
	
	/*
	 * Cleaner has to be shutdown before checkpointer because former calls
	 * the latter.
	 */
	before(EnvironmentImpl env) throws InterruptedException:
		execution(void EnvironmentImpl.shutdownDaemons()) && this(env){
		env.shutdownCleanerDaemon();
	}
	
	/**
	 * public for unit tests.
	 */
	public void EnvironmentImpl.shutdownCleanerDaemon() throws InterruptedException {

		if (cleanerDaemon != null) {
			cleanerDaemon.shutdown();
			cleanerDaemon=null;
		}
		return;
	}
	
	int around(EnvironmentImpl env):
		call(int EnvironmentImpl.invokeCleaner()) &&
		withincode(void UtilizationTracker.activateCleaner()) && target(env) {
		
		env.getCleanerDaemon().wakeup();
		return 0;
	}
	
	void around(UtilizationTracker ut) throws DatabaseException:
		execution(void UtilizationTracker.newPendingLN())&&this(ut){
		
	
		if (!ut.getEnvironment().getCleanerDaemon().areThreadsRunning()) {
			ut.activateCleaner();
		}
	}

}
