package com.sleepycat.je.cleaner;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.dbi.EnvironmentImpl;
import com.sleepycat.je.utilint.DaemonThread;

public class FileProcessorDaemon extends DaemonThread {
	
	private FileProcessor fileProcessor;
	private CleanerDaemon cleanerDaemon;


	FileProcessorDaemon(String name, EnvironmentImpl env, CleanerDaemon cleanerDaemon, FileProcessor fileProcessor) {
		super(0, name, env);
		this.fileProcessor = fileProcessor;
		this.cleanerDaemon=cleanerDaemon;
	}
	/**
	 * Return the number of retries when a deadlock exception occurs.
	 */
	protected int nDeadlockRetries() throws DatabaseException {

		return cleanerDaemon.nDeadlockRetries;
	}	
	
	/**
	 * Cleaner doesn't have a work queue so just throw an exception if it's ever
	 * called.
	 */
	public void addToQueue(Object o) throws DatabaseException {

		throw new DatabaseException(
				"Cleaner.addToQueue should never be called.");
	}
	
	
	/**
	 * Activates the cleaner. Is normally called when je.cleaner.byteInterval
	 * bytes are written to the log.
	 */
	public void onWakeup() throws DatabaseException {

		fileProcessor.doClean(true, // invokedFromDaemon
				true, // cleanMultipleFiles
				false); // forceCleaning
	}
	
	public void clearEnv() {
		fileProcessor.clearEnv();
	}
}
